<?php
$config=[];
require_once BASEPATH."../application/config/appbase.php";
$version=isset($config['app_version'])?$config['app_version']:'4.0.0';
AppInstaller::SetInstallerPath("installer");
AppInstaller::SetAppData("Support System Installer","Your complete support ticket system",$version);
AppInstaller::AddCss("css/custom.css");


AppInstaller::AddStep("step1");
AppInstaller::AddStep("step2");
AppInstaller::AddStep("step3");
AppInstaller::AddStep("step4");
AppInstaller::AddStep("finish");